"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getTracerInfo = exports.LUMIGO_ENDPOINT = void 0;
const api_1 = require("@opentelemetry/api");
// Setting the default Global logger to use the Console
// And optionally change the logging level (Defaults to INFO)
const LumigoHttpInstrumentation_1 = __importDefault(require("./instrumentros/LumigoHttpInstrumentation"));
const LumigoExpressInstrumentation_1 = __importDefault(require("./instrumentros/LumigoExpressInstrumentation"));
const sdk_trace_node_1 = require("@opentelemetry/sdk-trace-node");
const sdk_trace_base_1 = require("@opentelemetry/sdk-trace-base");
const instrumentation_1 = require("@opentelemetry/instrumentation");
const resources_1 = require("@opentelemetry/resources");
const exporter_collector_1 = require("@opentelemetry/exporter-collector");
const utils_1 = require("./utils");
const logLevel = (process.env.LUMIGO_DEBUG || 'false').toLowerCase() === 'true'
    ? api_1.DiagLogLevel.ALL
    : api_1.DiagLogLevel.ERROR;
api_1.diag.setLogger(new api_1.DiagConsoleLogger(), logLevel);
exports.LUMIGO_ENDPOINT = 'http://lumigo-wrapper-collector.golumigo.com:55681/v1/trace' || process.env.LUMIGO_ENDPOINT;
const getTracerInfo = () => {
    return utils_1.safeExecute(() => {
        let pkg;
        try {
            pkg = require('../package.json');
        }
        catch (e) {
            pkg = require('../../package.json');
        }
        const { name, version } = pkg;
        return { name, version };
    }, 'Failed to get wrapper version', 'warn', { name: '@lumigo/microservices-node-tracer', version: '0.0.0' })();
};
exports.getTracerInfo = getTracerInfo;
function requireIfAvailable(name) {
    try {
        require.resolve(name);
        require(name);
    }
    catch (e) {
        if (e.code === 'MODULE_NOT_FOUND') {
            console.warn(`module [${name}] not installed`);
        }
    }
}
const trace = (lumigoToken, serviceName, endpoint = exports.LUMIGO_ENDPOINT) => {
    try {
        if (process.env.LUMIGO_SWITCH_OFF && process.env.LUMIGO_SWITCH_OFF.toLowerCase() === 'true') {
            api_1.diag.debug('Lumigo is switched off');
            return;
        }
        const exporter = new exporter_collector_1.CollectorTraceExporter({
            // @ts-ignore
            serviceName,
            url: endpoint,
        });
        const config = {
            resource: new resources_1.Resource({
                lumigoToken: lumigoToken.trim(),
                'service.name': serviceName,
                runtime: `node${process.version}`,
                tracerVersion: exports.getTracerInfo().version,
                framework: 'express',
                envs: JSON.stringify(process.env),
            }),
        };
        const traceProvider = new sdk_trace_node_1.NodeTracerProvider(config);
        traceProvider.addSpanProcessor(new sdk_trace_base_1.BatchSpanProcessor(exporter));
        traceProvider.register();
        instrumentation_1.registerInstrumentations({
            instrumentations: [
                // @ts-ignore
                new LumigoHttpInstrumentation_1.default(lumigoToken, endpoint),
                // @ts-ignore
                new LumigoExpressInstrumentation_1.default(),
            ],
        });
        requireIfAvailable('express');
        requireIfAvailable('http');
        requireIfAvailable('https');
        api_1.diag.debug('Lumigo instrumentation started');
    }
    catch (e) {
        console.error('Lumigo tracer had an Error: ', e);
    }
};
if (process.env.LUMIGO_TOKEN && process.env.LUMIGO_SERVICE_NAME) {
    trace(process.env.LUMIGO_TOKEN, process.env.LUMIGO_SERVICE_NAME, process.env.LUMIGO_ENDPOINT || exports.LUMIGO_ENDPOINT);
}
module.exports = {
    trace,
    LumigoHttpInstrumentation: LumigoHttpInstrumentation_1.default,
    LumigoExpressInstrumentation: LumigoExpressInstrumentation_1.default,
    LumigoInstrumentations: (lumigoToken) => [
        new LumigoHttpInstrumentation_1.default(lumigoToken),
        new LumigoExpressInstrumentation_1.default(),
    ],
};
//# sourceMappingURL=wrapper.js.map